#!/bin/sh

# Extension.sh
#
# Script to install or uninstall InterMapper server extensions
#
# Usage:  Extension.sh { --install | --uninstall } file1 [ file2 ... ]

set -u

SDK_VERSION=1

SETTINGS_DIR=${SETTINGS_DIR:-"/var/local/InterMapper_Settings"}
EXTENSIONS_DIR=${EXTENSIONS_DIR:-"$SETTINGS_DIR/Extensions"}

PROGNAME=intermapperd

EOTHERERROR=100
ENOTINSTALLED=101
ECOPYFAILED=102
ENOPERMISSION=103

###################
# d o _ u s a g e #
###################
#
# Show script usage instructions.

do_usage ()
{
	test $# -eq 0 || echo "$1"
	echo "Usage: Extension.sh OPTION [FILE]..."
	echo "  Options:"
	echo "    --install"
	echo "    --uninstall"
	exit $EOTHERERROR
}

find_owner ()
{
	ls -lnd "$@" | awk {' if ($2 == "+") print $4; else print $3; '}
}

#######################
# d o _ i n s t a l l #
#######################
#
# Install extension files, set the appropriate permissions on those files, and
# tell intermapperd to reload the extensions.

do_install ()
{
	errcode=0
	errmsg=
	if [ ! -d "$SETTINGS_DIR/Probes" ] ; then
		echo "ERROR: SETTINGS_DIR $SETTINGS_DIR does not appear to be an InterMapper settings directory."
		exit $ENOTINSTALLED
	fi
		
	if [ ! -d "$EXTENSIONS_DIR" ]; then
		# Extensions dir is gone, try to re-create it.
		mkdir -p "$EXTENSIONS_DIR" 
		if [ "$?" != "0" ] ; then
			echo "ERROR: The extensions directory does not exist and we couldn't recreate it."
			exit $ENOPERMISSION
		fi

		imuser=`find_owner "$SETTINGS_DIR/Probes"`
		if [ "$?" != "0" ] ; then
			echo "ERROR: Something went wrong trying to determine the InterMapper user."
			exit $ENOPERMISSION
		fi

		chown "$imuser" "$EXTENSIONS_DIR" 2> /dev/null
		if [ "$?" != "0" ] ; then
			echo "ERROR: Something went wrong trying to change ownership of the Extensions directory to $imuser."
			exit $ENOPERMISSION
		fi
	fi

	if [ -d "$EXTENSIONS_DIR" ]; then
		# Fetch user name from owner of Extensions directory.
		imuser=`find_owner "$EXTENSIONS_DIR"`
		if [ "$?" != "0" -o "$imuser" = "" ] ; then
			ls -lnd $EXTENSIONS_DIR
			echo $imuser
			echo "ERROR: Couldn't determine the InterMapper user."
			exit $ENOPERMISSION
		fi
		
		# Install each file and set appropriate perms/owner.
		for fpath in "$@" ; do
			fname=`basename "$fpath"`
			echo "Copying $fname from " `dirname "$fpath"` " to $EXTENSIONS_DIR/$fname."
			temp_dest="$EXTENSIONS_DIR/$fname.XXXXXXXX"
			if [ -x "`which mktemp`" ] ; then
				temp_dest=`mktemp "$temp_dest"`
			fi
			cp "$fpath" "$temp_dest" > /dev/null 2>&1
			chmod 0444 "$temp_dest" > /dev/null 2>&1 
			chown "$imuser" "$temp_dest" > /dev/null 2>&1
			mv "$temp_dest" "$EXTENSIONS_DIR/$fname" > /dev/null 2>&1

			if [ ! -s "$EXTENSIONS_DIR/$fname" ] ; then
				echo "ERROR: Couldn't copy $fpath to $EXTENSIONS_DIR/$fname"
				exit $ENOTINSTALLED
			fi

			diff "$fpath" "$EXTENSIONS_DIR/$fname"
			if [ "$?" != "0" ] ; then
				echo "ERROR: failed to copy $fname to $EXTENSIONS_DIR."
				exit $ENOTINSTALLED
			fi

			whoownsit=`find_owner "$EXTENSIONS_DIR/$fname"`
			if [ "$whoownsit" != "$imuser" ]; then
				echo "ERROR: $EXTENSIONS_DIR/$fname is not owned by $imuser (was owned by $whoownsit instead)."
				exit $ENOPERMISSION
			fi
		done
		
		# Tell intermapperd to reload extensions; ignore result if it's not running. 

		killall -SIGHUP "$PROGNAME" || true

		return 0
	fi
	
	return $EOTHERERROR
}


###########################
# d o _ u n i n s t a l l #
###########################
#
# Uninstall extension files bearing the same basename as the arguments, then
# tell intermapperd to reload the extensions.

do_uninstall ()
{
	errcode=0
	if [ -d "$EXTENSIONS_DIR" ]; then
		# Delete each file name in the Extensions directory that matches.
		for fpath in "$@" ; do
			fname=`basename "$fpath"`
			if [ -f "$EXTENSIONS_DIR/$fname" ]; then
				rm -f "$EXTENSIONS_DIR/$fname"
				if [ "$?" != "0" ] ; then
					echo "Failed to remove $EXTENSIONS_DIR/$fname."
					errcode=$ENOPERMISSION
				fi
			fi
		done
		
		# Tell intermapperd to reload extensions; ignore result if it's not running. 

		killall -SIGHUP "$PROGNAME" || true
			
	else
		echo "$EXTENSIONS_DIR is not a directory!?"
		exit $ENOTINSTALLED
	fi
	
	return $errcode
}


###############################
# d o _ s d k _ v e r s i o n #
###############################
#
# Echo the SDK version and return OK.

do_sdk_version ()
{
	echo "${SDK_VERSION}"
	return 0
}

## Begin Main Script ##

if [ "`basename $0`" = "Extension.sh" ]; then
	if [ $# -lt 1 ]; then
		do_usage "$0: Too few arguments."
	fi

	opt="$1"
	shift
	cd /

	case $opt in 
		--install )		do_install "$@" ;;
		--uninstall )	do_uninstall "$@" ;;
		--sdk-version ) do_sdk_version ;;
		--help )		do_usage ;;

		* )	do_usage "$0: Illegal option $opt" ;;
	esac
	exit $?
fi

## End Main Script ##
